package TestUtil;
use warnings;
use strict;

use Fcntl;

use base 'Exporter';

our @EXPORT = qw/assert_cleaned_html assert_cleaned_html_spawn assert_cleaned_css assert_cleaned_css_decl/;

use Test::More;

use lib './third-party/HTML-Tree-3.23/lib';
use lib './third-party/CSS-SAC-0.08/blib/lib';
use lib './third-party/Class-ArrayObjects-1.03/lib';
use lib '.';

use CleanHTML;
use CleanCSS;

use HTML::TreeBuilder;

# assert_cleaned_html(label, dirty_html [, expect_html])
sub assert_cleaned_html {
    my ($test_label, $dirty_html, $expect_html, $spawn_p) = @_;
    if (defined $expect_html and $expect_html !~ /^<!DOCTYPE/) {
        if ($expect_html !~ /<html/i) {
            # implicit tags
            $expect_html =
                qq{<html><head><meta http-equiv="Content-Type" } .
                qq{content="text/html; charset=us-ascii">}.
                qq{</head><body>$expect_html</body></html>};
        }
        $expect_html = "<!DOCTYPE HTML>\n" . $expect_html;
    }
    my $clean_html;
    if ($spawn_p) {
        open my $tempfile, '>', undef;
        print $tempfile $dirty_html;
        my $tempfileno = fileno($tempfile);
        my $old_flags = fcntl($tempfile, F_GETFD, 0);
        fcntl($tempfile, F_SETFD, $old_flags & ~FD_CLOEXEC) or die $!;
        $clean_html = `perl -T htmlfilter.perl <&$tempfileno`;
        close $tempfile;
    } else {
        $clean_html = clean_html($dirty_html);
    }

    if (defined $expect_html) {
        is($clean_html, $expect_html, "$test_label matches expected");
    }
    
    my $tree = HTML::TreeBuilder->new;
    $tree->parse($clean_html);
    $tree->eof();

    #diag("Result of $test_label = $clean_html");

    ok( !$tree->look_down( _tag => 'script' ),
        "$test_label contains no script tags" );
    ok( !$tree->look_down(
            sub { ref $_[0] and grep { $_ =~ /^on/i } $_[0]->all_external_attr_names } ),
        "$test_label contains no attributes whose names start with on" );
    # these will have to change if our filtering gets smart enough to distinguish
    # safe and unsafe tags of these types.
    ok( !$tree->look_down( _tag => 'img' ),
        "$test_label contains no img tags" );
    ok( !$tree->look_down( _tag => 'link' ),
        "$test_label contains no link tags" );
    ok( !$tree->look_down( _tag => 'applet' ),
        "$test_label contains no applet tags" );
    ok( !$tree->look_down( _tag => 'embed' ),
        "$test_label contains no embed tags" );
    ok( !$tree->look_down( _tag => 'object' ),
        "$test_label contains no object tags" );

    # generally we shouldn't and don't filter all occurences of expression
    # and url( in stylesheets (e.g. not those in strings) but our tests
    # should never rely on that. We should have more thorough tests for stylesheet
    # safety anyways.
    ok( !$tree->look_down( style => qr/expression/ ),
        "$test_label contains no style attributes containing expression" );
    ok( !$tree->look_down( style => qr/url\(/ ),
        "$test_label contains no style attributes containing expression" );
    ok( !$tree->look_down( _tag => 'style', src => qr/./ ),
        "$test_label contains a style tag with a src attribute" );

    $tree->delete; 

    is($clean_html, clean_html($clean_html),
       "double-cleaning $test_label HTML is a no-op");
}

# assert_cleaned_css(label, dirty_css, clean_css)
sub assert_cleaned_css {
    my ($label, $dirty_css, $expect_css) = @_;
    my $clean_css = clean_css($dirty_css);
    #diag("$label cleaned = $clean_css");
    if (ref $expect_css) {
        like($clean_css, $expect_css, $label);
    } else {
        is($clean_css, $expect_css, $label);
    }
}

# assert_cleaned_css_decl(label, dirty_css, clean_css)
sub assert_cleaned_css_decl {
    my ($label, $dirty_css, $expect_css) = @_;
    my $clean_css = clean_css_decl($dirty_css);
    is($clean_css, $expect_css, $label);
    #diag("$label cleaned = $clean_css");
    assert_cleaned_css(
        "$label as stylesheet",
        "*{$dirty_css}",
        qr/^(?: \* \{ \Q$expect_css\E\} |)\z/);
}

sub assert_cleaned_html_spawn {
    my ($label, $dirty_html, $expect_html) = @_;
    assert_cleaned_html($label, $dirty_html, $expect_html, 1);
}

1;
