#!/usr/bin/perl
use warnings;
use strict;
use TestUtil;
use Test::More 'no_plan';

# All the following examples from http://ha.ckers.org/xss.html :

assert_cleaned_html('document.wriute("<SCRI"',
    <<EOS, 'PT SRC=&#34;http://yourserver.com/evilness.js&#34;&#62; ');
<SCRIPT>document.write("<SCRI");</SCRIPT>PT 
SRC="http://yourserver.com/evilness.js"></SCRIPT>
EOS
assert_cleaned_html('ha.ckers.org/xss.html locator 2',
                    q/'';!--"<XSS>=&{()}/, q/&#39;&#39;;!--&#34;=&#38;{()}/);
assert_cleaned_html('img src trick',
                    q/<IMG SRC="javascript:alert('XSS');">/, '[image]');
assert_cleaned_html('img src trick (2)',
                    q/<IMG SRC=javascript:alert('XSS');>/, '[image]');
assert_cleaned_html('img src trick (3)',
                    q/<IMG SRC=JaVaScRiPt:alert('XSS');>/, '[image]');
assert_cleaned_html('img src trick (4)',
                    q/<IMG SRC=javascript:alert(&quot;XSS&quot;);>/, '[image]');
assert_cleaned_html('img src trick (5)',
                    q/<IMG SRC=`javascript:alert(&quot;XSS&quot;);`>/, '[image]');
assert_cleaned_html('malformed tag',
                    q{<IMG """><SCRIPT>alert("XSS")</SCRIPT>">}, q{[image]&#34;&#62;});
assert_cleaned_html('strange entities',
                    q{<IMG SRC=&#0000106&#0000097&#0000118&#0000097&#0000115&#0000099&#0000114&#0000105&#0000112&#0000116&#0000058&#0000097&#0000108&#0000101&#0000114&#0000116&#0000040&#0000039&#0000088&#0000083&#0000083&#0000039&#0000041>},
                    q{[image]});
assert_cleaned_html('strange entities (attribute after in a tag)',
                    q{<a href=&#0000106&#0000097&#0000118&#0000097&#0000115&#0000099&#0000114&#0000105&#0000112&#0000116&#0000058&#0000097&#0000108&#0000101&#0000114&#0000116&#0000040&#0000039&#0000088&#0000083&#0000083&#0000039&#0000041 onmouseover=xxx>text},
                    q{<a href="javascript:alert(&#39;XSS&#39;)">text</a>});
assert_cleaned_html('<scr\\0ipt>',
                    qq{<SCR\0IPT>alert("TEST")</SCR\0IPT>},
                    q{alert(&#34;TEST&#34;)});

assert_cleaned_html('<script/foo>',
                    qq{<SCRIPT/FOO SRC="xxx"></SCRIPT>},
                    q{});
assert_cleaned_html('on...!#$%&()*~=...',
                    q{<BODY onload!#$%&()*~+-_.,:;?@[/|\]^`=alert("XSS")>},
                    q{});
assert_cleaned_html('<script/src=...>',
                    qq{<SCRIPT/SRC="http://failed/">\n</SCRIPT>},
                    q{});
assert_cleaned_html('<<SCRIPT>',
                    qq{<<SCRIPT>alert("XSS");//<</SCRIPT>},
                    q{&#60;});
assert_cleaned_html('<SCRIPT SRC=...<B>',
                    qq{<SCRIPT SRC=http://foo/?<B>},
                    q{});
assert_cleaned_html('<IMG SRC="..." [unclosed]',
                    qq{<IMG SRC="javascript:alert('XSS')"},
                    q{});
assert_cleaned_html('<iframe src=... <',
                    qq{<iframe src=http://bar/ <},
                    q{});
assert_cleaned_html('<input type="image"',
                    q{<INPUT TYPE="IMAGE" SRC="javascript:alert('XSS');">},
                    q{<input>});
assert_cleaned_html('<body background',
                    q{<BODY BACKGROUND="javascript:alert('XSS')">},
                    q{});
assert_cleaned_html('<bgsound src=',
                    q{<BGSOUND SRC="javascript:alert('XSS');">},
                    q{});
assert_cleaned_html('&{script} (1)',
                    q/<BR SIZE="&{alert('XSS')}">/,
                    q{<br>});
assert_cleaned_html('&{script} (2)',
                    q{&{alert('XSS')}},
                    q{&#38;{alert(&#39;XSS&#39;)}});
assert_cleaned_html('link rel=stylesheet',
                    q{<LINK REL="stylesheet" HREF="hostile.css">},
                    q{});

assert_cleaned_css('@import',
                   q{@import'http://eve.com';},
                   q{});

assert_cleaned_html('<meta http-equiv=link',
                    q{<META HTTP-EQUIV="Link" Content="<XXX>; REL=stylesheet">},
                    q{});
assert_cleaned_css_decl('-moz-binding', '-moz-binding: url(eve.xul)', '');
assert_cleaned_css_decl('behavior', 'behavior: url(eve.htc)', '');
assert_cleaned_css_decl('no end quote', 
                        q{background: url("javascript:alert('XSS'))}, q{});
assert_cleaned_css_decl('list-style-image', 
                        q{list-style-image: url("javascript:alert('XSS')")}, q{});
assert_cleaned_html('8-bit chars',
                    "\xbcscript\xbealert(\xa2XSS\xa2);\xbc/script\xbe",
                    q{&#188;script&#190;alert(&#162;XSS&#162;);&#188;/script&#190;});
assert_cleaned_html('meta refresh',
                    q{<META HTTP-EQUIV="refresh" CONTENT="0;url=data:text/html;base64,danger">},
                    q{});
assert_cleaned_html('frameset',
                    q{<FRAMESET><FRAME SRC="javascript:alert('XSS');"></FRAMESET>},
                    q{});
assert_cleaned_html('table background',
                    q{<TABLE BACKGROUND="javascript:alert()">},
                    q{<table></table>});

assert_cleaned_css_decl('escaping',
                        q{background:\0075\0072\006C\0028'\006a\0061\0076\0061\0073\0063\0072\0069\0070\0074\003a\0061\006c\0065\0072\0074\0028.1027\0058.1053\0053\0027\0029'\0029},
                        q{});
assert_cleaned_css_decl('\\1',
                        qq{background:url(\x01javascript:alert('XSS'))},
                        q{});
assert_cleaned_css_decl('expression()',
                        qq{width: expression(alert('XSS'))},
                        q{});
assert_cleaned_css_decl('escaped import',
                        q{@im\port'\ja\vasc\ript:alert("XSS")';},
                        q{});
assert_cleaned_css_decl('expression() with malformed comment',
                        qq{width: expr/*XSS*\n/ession(alert('XSS'))},
                        q{});

assert_cleaned_html('style type=text/javascript',
                    q{<HTML><HEAD><STYLE TYPE="text/javascript">alert('XSS');</STYLE></HEAD>
                      <BODY></BODY></HTML>},
                    q{<html><head><meta http-equiv="Content-Type" content="text/html; charset=us-ascii">}
                    .q{<style></style></head><body></body></html>});
assert_cleaned_html('IE comment',
                    qq{<!--[if gte IE 4]>\n<SCRIPT>alert('XSS');</SCRIPT>\n<![endif]-->},
                    q{});
assert_cleaned_html('BASE HREF=javascript',
                    q{<BASE HREF="javascript:alert('XSS');//">},
                    q{});
assert_cleaned_html('XML namespace implemention',
                    q{<HTML xmlns:xss>
                      <?import namespace="xss" implementation="http://eve.com/">},
                    q{});
assert_cleaned_html('XML CDATA bizareness',
                    q{
<XML ID=I><X><C><![CDATA[<IMG SRC="javas]]><![CDATA[cript:alert('XSS');">]]>
</C></X></xml><SPAN DATASRC=#I DATAFLD=C DATAFORMATAS=HTML></SPAN>
},
                    q{]]&#62; <span></span>});
assert_cleaned_html('XML comment bizareness',
                    q{
<XML ID="xss"><I><B>&lt;IMG SRC="javas<!-- -->cript:alert('XSS')"&gt;</B></I></XML>
<SPAN DATASRC="#xss" DATAFLD="B" DATAFORMATAS="HTML"></SPAN>},
                    q{<i><b>&#60;IMG SRC=&#34;javascript:alert(&#39;XSS&#39;)&#34;&#62;</b></i> }
                    .q{<span></span>});
assert_cleaned_html('urn:schemas-microsoft-com:time',
                    q{
<HTML><BODY>
<?xml:namespace prefix="t" ns="urn:schemas-microsoft-com:time">
<?import namespace="t" implementation="#default#time2">
<t:set attributeName="innerHTML" to="XSS&lt;SCRIPT DEFER&gt;alert(&quot;XSS&quot;)&lt;/SCRIPT&gt;">
</BODY></HTML>
                    },
                    q{});
assert_cleaned_html('UTF-7',
                    q{<HEAD><META HTTP-EQUIV="CONTENT-TYPE" CONTENT="text/html; charset=UTF-7"> </HEAD>+ADw-SCRIPT+AD4-alert('XSS');+ADw-/SCRIPT+AD4-},
                    q{+ADw-SCRIPT+AD4-alert(&#39;XSS&#39;);+ADw-/SCRIPT+AD4-});

assert_cleaned_html_spawn('UTF-7 [external processs]',
                    q{<HEAD><META HTTP-EQUIV="CONTENT-TYPE" CONTENT="text/html; charset=UTF-7"> </HEAD>+ADw-SCRIPT+AD4-alert('XSS');+ADw-/SCRIPT+AD4-},
                    qq{<!DOCTYPE HTML>\n} .
                    q{<html><head><meta http-equiv="Content-Type" content="text/html; charset=us-ascii">} .
                    q{</head><body></body></html>});
